%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The main file for the uniformity and the quantum efficiency test    %
% It does automatic processing of data files. A list of tubes'        %
% serial numbers must be saved an a "automation" file each SN on      %
% a separate line. Data files should be placed in a subdirectory      %
% called "data" of the directory in which this script is located.     %
% Each file name should follow the following naming conventions:      %
%                                                                     %
% serial number + S.txt - for surface scan data files                 %
%  or:                                                                %
% serial number + QE.txt - for quantum efficiency data files          %
%                                                                     %
% Author: Mihai Cara, mcara@purdue.edu                                %
% Last Updated by Mihai Cara on 01/01/2005                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;
fclose all;
clc;

% define constants:
delta_r = 0.5; %the increment used in the radial profile
% delta_sig = 3.6;%the increment used in the azimuthal profile
delta_sig = 10;%the increment used in the azimuthal profile
az_radius = 12.5; %the efective radius 

% Optimization constants:
MaxIter = 2000; % input the number of iterations (for a better accuracy use more 
                % than 250 iterations)
Weighted = 1; % 1 - weighted, 0 - without weights

% OPTIONS:
savesummaryfiles = 0; % set =1 if you want to get a summary file for each PMT
savefig      = 3; % save the plot:
                  % 0 - do not save;
                  % 1 - save in PNG format
                  % 2 - save in MATLAB FIG format
                  % 3 - save both PNG and FIG formats
% NA_tb_cell_da = 'N/A'; % type the text you want to appear in table cells
                       % if data are not available
                       % use '&nbsp;' is you want to leave the cells empty
NA_tb_cell_da = '&nbsp;';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
current_mdir = pwd; % find current matlab directory
addpath(strcat(current_mdir,'\functions')); % add functions to search path
% build root directory name:
pos = strfind(current_mdir,'\');
root_dir = current_mdir(1:pos(size(pos, 2))-1);
% get batch "data" directory:
batch_dir = uigetdir(strcat(root_dir, '\data'), 'Select batch data directory');
if batch_dir == 0
    return;
end
pos = strfind(batch_dir,'\');
batchno = batch_dir(pos(size(pos, 2))+1:end); % extract batch number from directory name
%
sn_info_type = 0; % - none (use dir list); 1 - snlist present; 2 - factory data present
% 
dir1 = [batch_dir '\summary'];
if (exist(dir1, 'dir') ~= 7)
    mkdir(batch_dir, 'summary');
end
% check that factory data are present and read the file:
fname1 = [batch_dir '\' batchno '.txt'];
fname2 = [batch_dir '\summary\' batchno '.txt'];
fname3 = [batch_dir '\snlist.txt'];
fname4 = [batch_dir '\summary\snlist.txt'];

if (exist(fname2, 'file') == 2)
    sn_info_type = 2;
    fname = fname2;
elseif (exist(fname1, 'file') == 2)
    sn_info_type = 2;
    fname = fname2;
    movefile(fname1, fname2); % move factory data file to summary directory
end
if (exist(fname4, 'file') == 2)
    if sn_info_type == 0
        sn_info_type = 1;
        fname = fname4;
    end
elseif (exist(fname3, 'file') == 2)
    if sn_info_type == 0
        sn_info_type = 1;
        fname = fname4;
    end
    movefile(fname3, fname4); % move factory data file to summary directory
end

switch sn_info_type
    case 0
        [type1,type2,date1,date2,batch_no1,batch_no2,...
                batch_size1,batch_size2,...
                snlist,cath_blue,hv_factory,dark,pv] = get_snlist(batchno, batch_dir, NA_tb_cell_da);
        save_snlist(fname4, snlist);
    case 1
        [type1,type2,date1,date2,batch_no1,batch_no2,...
                batch_size1,batch_size2,...
                snlist,cath_blue,hv_factory,dark,pv] = read_snlist_file(batchno, fname, NA_tb_cell_da);
    case 2
        [type1,type2,date1,date2,batch_no1,batch_no2,...
                batch_size1,batch_size2,...
                snlist,cath_blue,hv_factory,dark,pv] = read_fact_file(batchno, fname, NA_tb_cell_da);
        if (exist(fname4, 'file') ~= 2)
            save_snlist(fname4, snlist);
        end
end

% read the HTML template files:
fd = fopen('templates\1.htm'); htm1 = fscanf(fd,'%c'); fclose(fd);
fd = fopen('templates\2.htm'); htm2 = fscanf(fd,'%c'); fclose(fd);
fd = fopen('templates\3.htm'); htm3 = fscanf(fd,'%c'); fclose(fd);

% create necessary subdirectories:
if ((savefig == 1 || savefig == 3) && exist([root_dir '\png'], 'dir') ~= 7)
    mkdir(root_dir, 'png');
end
if ((savefig == 2 || savefig == 3) && exist([root_dir '\fig'], 'dir') ~= 7)
    mkdir(root_dir, 'fig');
end
if (exist([root_dir '\html'], 'dir') ~= 7)
    mkdir(root_dir, 'html');
end
if (exist([root_dir '\html\data'], 'dir') ~= 7)
    mkdir([root_dir '\html'], 'data');
end

if (exist([root_dir '\png\' batchno], 'dir') == 7)
    rmdir([root_dir '\png\' batchno], 's');
end
mkdir([root_dir '\png'], batchno);

if (exist([root_dir '\fig\' batchno], 'dir') == 7)
    rmdir([root_dir '\fig\' batchno], 's');
end
mkdir([root_dir '\fig'], batchno);

if (exist([root_dir '\html\data\' batchno], 'dir') == 7)
    rmdir([root_dir '\html\data\' batchno], 's');
end
mkdir([root_dir '\html\data\'], batchno);

fdallsummary = fopen([batch_dir '\summary\summary.txt'], 'w');
fprintf(fdallsummary, ['Serial No.:\tQE Performed:\t'...
        'Average QE over 310-450nm, (%%):\tSurf. scan performed:\t'...
        'Gain:\tHV, (V):\tEff. radius, (mm):\t'...
        'Non-uniformity, (%%):\tSensetivity slope, (1/mm):\r\n\r\n']);

sntested = zeros(1, size(snlist, 2));
qeperformed   = {};
surfperformed = {};
gains  = {};
avqes  = {};
effrs  = {};
nunifs = {};
slopes = {};
hvs    = {};

for k = 1:size(snlist,2)
    serialno  = snlist{k};
	gains{k}  = NA_tb_cell_da;
	avqes{k}  = NA_tb_cell_da;
	effrs{k}  = NA_tb_cell_da;
	nunifs{k} = NA_tb_cell_da;
	slopes{k} = NA_tb_cell_da;
	hvs{k}    = NA_tb_cell_da;
    qeperformed{k}   = 'No';
	surfperformed{k} = 'No';

    is_pamunif = 'No';
    is_qe      = 'No';
    % create surface scan file name:
    surfname = [batch_dir '\' serialno 'S.txt'];
    qename = [batch_dir '\' serialno 'QE.txt'];
    htmlname = [root_dir '\html\data\' batchno '\' serialno '.htm'];
    if (exist(surfname, 'file') == 2)
        fdsurf = fopen(surfname);
        file_ver = fscanf(fdsurf, 'File Version = %d\n'); %reads the file version
        test_mode = fscanf(fdsurf, 'Test Mode: %d'); %reads the test mode
        s = fgetl(fdsurf); % file version - the rest of the line
        if (file_ver < 1 | file_ver > 4)
            fclose(fdsurf);
            error('This version not supported yet!')
            continue;
        end
        if (test_mode ~= 2)
            fclose(fdsurf);
            error('File naming and test mode mismatch!')
            continue;
        end
        [v, nunif, fighs, UserID, TestDate, TubeManufacturer,...
                TubeModel, SerialNumber, TubeGain, HighVoltage, LSPower, atten] = ...
            pamunif(fdsurf,file_ver,delta_r,delta_sig,az_radius,MaxIter,Weighted,[4,10,11,12]);
        is_pamunif = 'Yes';

        % save the plot:
        saveplot(fighs, root_dir, batchno, serialno, 'S', savefig, 1);
    end
    if (exist(qename, 'file') == 2)
        fdqe = fopen(qename);
        file_ver = fscanf(fdqe, 'File Version = %d\r\n'); %reads the file version
        test_mode = fscanf(fdqe, 'Test Mode: %d'); %reads the test mode
        s = fgetl(fdqe); % file version - the rest of the line
        if (file_ver < 1 | file_ver > 4)
            fclose(fdqe);
            error('This version not supported yet!')
            continue;
        end
        if (test_mode ~= 1)
            fclose(fdqe);
            error('File naming and test mode mismatch!')
            continue;
        end
        [avqe, fighqe, UserID, TestDate, TubeManufacturer,...
                TubeModel, SerialNumber, TubeGain, HighVoltage, LSPower] = qe(fdqe, file_ver);
        is_qe = 'Yes';

        % save the plot:
        saveplot(fighqe, root_dir, batchno, serialno, 'QE', savefig, 1);
    end
    qeperformed{k}   = is_qe;
    surfperformed{k} = is_pamunif;

    if (strcmp(is_qe, 'No') && strcmp(is_pamunif, 'No'))
        fprintf(fdallsummary, '%s\t%s\tN/A\t%s\tN/A\tN/A\tN/A\tN/A\tN/A\r\n',...
            serialno, is_qe, is_pamunif);

        % save brief summary in a separate text file:
        if (savesummaryfiles ==1)
            fdsummary = fopen([batch_dir '\summary\' serialno '_info.txt'], 'w');
            fprintf(fdsummary, ['No test was performed on PMT with SN: %s\r\n'...
                    'The PMT was either defective or did not perform well in'...\
                    ' the single photon peak test.'], serialno);
            fclose(fdsummary);
        end
        continue
    end
    sntested(k) = 1; % a test on the tube was performed
    % overwrite values from surface scan (QE has priority especially
    % for gain and HV)
    sn = sscanf(SerialNumber, 'Serial Number: %s');
    pam_gain = sscanf(TubeGain, 'Tube Gain:%g');
    hvoltage = sscanf(HighVoltage, 'High Voltage, (V):%g');
    hvs{k} = num2str(hvoltage);

    % save brief summary in 'summary.txt'
    fprintf(fdallsummary, '%s\t%s\t', sn, is_qe);
    testmodes = 0;
    if (strcmp(is_qe, 'Yes'))
        testmodes = testmodes + 2;
        fprintf(fdallsummary, '%.3g\t', avqe);
        avqes{k} = sprintf('%.3g', avqe);
    else
        fprintf(fdallsummary, 'N/A\t');
    end
    if (strcmp(is_pamunif, 'Yes'))
        testmodes = testmodes + 1;
        fprintf(fdallsummary, '%s\t%g\t%g\t%.4g\t%.3g\t%g\r\n', is_pamunif, pam_gain,...
            hvoltage, v(3), nunif, v(2));
    else
        fprintf(fdallsummary, '%s\tN/A\tN/A\tN/A\tN/A\tN/A\r\n', is_pamunif);
    end
    
    % save brief summary in a separate text file:
    if (savesummaryfiles ==1)
		fdsummary = fopen([batch_dir '\summary\' serialno '_info.txt'], 'w');
		fprintf(fdsummary, '%s\r\n%s\r\n%s\r\n%s\r\n%s\r\n', TubeModel, SerialNumber, TubeGain,...
            HighVoltage, TestDate);
		if (strcmp(is_pamunif, 'Yes'))
            fprintf(fdsummary, ['\r\nNon-uniformity, (%%): %.3g\r\n'...
                    'Sensitivity slope at edge, (1/mm): %g\r\n'...
                    'Effective radius, (mm): %.4g\r\n'...
                    'QE attenuation factor (at %.3g, %.3g, %.3g mm): %.3g, %.3g, %.3g\r\n'], nunif, v(2), v(3),...
                atten(1,1), atten(2,1), atten(3,1), atten(1,2), atten(2,2), atten(3,2));
		else
            fprintf(fdsummary, ['\r\nNon-uniformity, (%%): N/A\r\n'...
                    'Sensitivity slope at edge, (1/mm): N/A\r\n'...
                    'Effective radius, (mm): N/A\r\n'...
                    'QE attenuation factor: N/A\r\n']);
		end
		
		if (strcmp(is_qe, 'Yes'))
            fprintf(fdsummary, 'Average QE (380-450nm), (%%): ~%.3g', avqe);
		else
            fprintf(fdsummary, 'Average QE (380-450nm), (%%): N/A');
		end

        fclose(fdsummary);
    end
    % write HTML file:
    fdhtml = fopen(htmlname, 'w');
    shortserial = sscanf(SerialNumber, 'Serial Number: %s');
    gain = sscanf(TubeGain, 'Tube Gain: %g');
    gainexp = floor(log10(gain));
    gain = gain/realpow(10, gainexp);
    gains{k} = sprintf('%.3g10<sup>%d', gain, gainexp);
    if (strcmp(is_pamunif, 'Yes'))
        effrs{k}  = sprintf('%.4g', v(3));
        slopes{k} = sprintf('%.3g', v(2));
        nunifs{k} = sprintf('%.3g', nunif);
    end

    datestring = get_last_update; % create current date string
            
    switch testmodes
        case 1
            fprintf(fdhtml, htm1,...
                shortserial, shortserial, shortserial, ...
                TubeModel, SerialNumber, gain, gainexp, HighVoltage, TestDate,...
                nunif, v(2), v(3),...
                atten(1,1), atten(2,1), atten(3,1), atten(1,2), atten(2,2), atten(3,2),...
                batchno, serialno, batchno, serialno, batchno, serialno, datestring);
        case 2
            fprintf(fdhtml, htm2,...
                shortserial, shortserial, shortserial, ...
                TubeModel, SerialNumber, gain, gainexp, HighVoltage, TestDate, avqe,...
                batchno, serialno, batchno, serialno, batchno, serialno, datestring);
        case 3
            fprintf(fdhtml, htm3,...
                shortserial, shortserial, shortserial, ...
                TubeModel, SerialNumber, gain, gainexp, HighVoltage, TestDate,...
                nunif, v(2), v(3),...
                atten(1,1), atten(2,1), atten(3,1), atten(1,2), atten(2,2), atten(3,2), avqe,...
                batchno, serialno, batchno, serialno, batchno, serialno,...
                batchno, serialno, batchno, serialno, batchno, serialno, datestring);
    end
    fclose(fdhtml);
end
save_list_html(batchno, root_dir, current_mdir, snlist, sntested, sn_info_type);
save_summary_html(batchno, root_dir, current_mdir, ...
    snlist, qeperformed, surfperformed, avqes, gains, hvs, effrs, nunifs, slopes, ...
    type1, type2, date1, date2, batch_no1, batch_no2, batch_size1, batch_size2, ...
    cath_blue,hv_factory,dark,pv);
fclose(fdallsummary);
fclose all;
clear all;